@@ -30,6 +30,7 @@ static GPdfURIInput *uri_in;
 /* The list of all open windows */
 static GList *window_list = NULL;
 
+static void gpdf_window_update_window_title (GPdfWindow *);
 
 static gboolean
 create_window (gpointer data)
@@ -103,11 +104,11 @@ open_request_handler (GPdfURIInput *in, const char *uri, gpointer user_data)
 
 	window = find_empty_or_create_new_window (window_list);
 	if (gpdf_window_open (window, uri)) {
+	  	gpdf_window_update_window_title (window);
 		gtk_widget_show_now (GTK_WIDGET (window));
-	} else {
-		if (g_list_length (window_list) > 1)
-			gtk_widget_destroy (GTK_WIDGET (window));
-	}
+	} else
+	  if (g_list_length (window_list) > 1)
+	    	gtk_widget_destroy (GTK_WIDGET (window));
 }
 
 #ifdef PDF_DEBUG
@@ -226,35 +227,48 @@ gw_add_control_to_ui (GPdfWindow *gpdf_window)
 
 	g_object_ref (G_OBJECT (bonobo_widget));
 	priv->bonobo_widget = BONOBO_WIDGET (bonobo_widget);
-
-	bonobo_control_frame_control_activate (
-		bonobo_widget_get_control_frame (BONOBO_WIDGET (bonobo_widget)));
-
-	gtk_widget_show (bonobo_widget);
 	gtk_container_add (GTK_CONTAINER (priv->slot), bonobo_widget);
-
 	CORBA_exception_free (&ev);
 
 	return TRUE;
 }
 
-static Bonobo_Stream
-bonobo_stream_for_uri (const char *uri, CORBA_Environment *ev)
+static void
+gpdf_window_update_window_title (GPdfWindow *gpdf_window)
 {
-	char *escaped_uri;
-	char *moniker_str;
-	Bonobo_Stream stream;
-
-	g_return_val_if_fail (uri != NULL, CORBA_OBJECT_NIL);
-	g_return_val_if_fail (ev != NULL, CORBA_OBJECT_NIL);
-
-	escaped_uri = gnome_vfs_escape_set (uri, "!");
-	moniker_str = g_strconcat ("vfs:", escaped_uri, NULL);
-	stream = bonobo_get_object (moniker_str, "IDL:Bonobo/Stream:1.0", ev);
-	g_free (moniker_str);
-	g_free (escaped_uri);
-
-	return stream;
+	Bonobo_PropertyBag pb;
+	CORBA_Environment ev;
+	char *exception_str;
+	gboolean is_default;
+	char *title;
+	
+	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window));
+	
+	CORBA_exception_init (&ev);
+	
+	pb = bonobo_control_frame_get_control_property_bag (
+		bonobo_widget_get_control_frame (
+			gpdf_window->priv->bonobo_widget), &ev);
+	
+	/* Here we handle a exception when getting PB */
+	if (BONOBO_EX (&ev)) {
+		/* Report exception & use default title */
+		exception_str = bonobo_exception_get_text (&ev);
+		g_warning ("Property bag access exception '%s'", exception_str);
+		g_free (exception_str);
+		title = g_strdup (_("PDF Document"));
+	} else {
+		title = bonobo_pbclient_get_string_with_default (pb, "title",
+								 _("PDF Document"),
+								 &is_default);
+		bonobo_object_release_unref (pb, &ev);
+	}
+	
+	CORBA_exception_free (&ev);
+	
+	g_object_set (G_OBJECT (gpdf_window), "title", title, NULL);
+	
+	g_free (title);
 }
 
 static gboolean
@@ -263,8 +277,7 @@ gw_control_load_pdf (GPdfWindow *gpdf_window, const char *uri)
 	GPdfWindowPrivate *priv;
 	CORBA_Environment ev;
 	Bonobo_Unknown control;	
-	Bonobo_PersistStream persist;
-	Bonobo_Stream stream;
+	Bonobo_PersistFile persist;
 	gboolean success = FALSE;
 
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
@@ -275,29 +288,23 @@ gw_control_load_pdf (GPdfWindow *gpdf_window, const char *uri)
 	g_return_val_if_fail (control != CORBA_OBJECT_NIL, FALSE);
 	
 	CORBA_exception_init (&ev);
+
 	persist = Bonobo_Unknown_queryInterface (
-		control, "IDL:Bonobo/PersistStream:1.0", &ev);
-	
+		control, "IDL:Bonobo/PersistFile:1.0", &ev);
 
 	if (BONOBO_EX (&ev) || persist == CORBA_OBJECT_NIL) {
 		gnome_error_dialog (
-			"Panic: component doesn't implement PersistStream.");
+			"Panic: component doesn't implement PersistFile.");
 		goto error_persist;
 	}
 
-	stream = bonobo_stream_for_uri (uri, &ev);
+	Bonobo_PersistFile_load (persist, uri, &ev);
 
-	if (BONOBO_EX (&ev) || stream == CORBA_OBJECT_NIL) {
-		char *err = g_strdup_printf (_("Could not open %s"), uri);
-		gnome_error_dialog /* FIXME _parented */ (err);
-		g_free (err);
+	if (BONOBO_EX (&ev))
 		goto error_stream;
-	}
 
-	Bonobo_PersistStream_load (persist, stream, "application/pdf", &ev);
-	success = TRUE;
+	success = TRUE;		
 
-	bonobo_object_release_unref (stream, &ev);
 error_stream:
 	bonobo_object_release_unref (persist, &ev);
 error_persist:
@@ -309,13 +316,30 @@ error_persist:
 gboolean
 gpdf_window_open (GPdfWindow *gpdf_window, const char *name)
 {
+	GPdfWindowPrivate *priv;
+
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
 	g_return_val_if_fail (!gpdf_window_has_contents (gpdf_window), FALSE);
 
+	priv = gpdf_window->priv;
+
 	if (gw_add_control_to_ui (gpdf_window) == FALSE)
 		return FALSE;
 
-	return gw_control_load_pdf (gpdf_window, name);
+	if (gw_control_load_pdf (gpdf_window, name) == FALSE) {
+	  	gtk_container_remove (GTK_CONTAINER (priv->slot),
+				      GTK_WIDGET (priv->bonobo_widget));
+		gtk_widget_destroy (GTK_WIDGET (priv->bonobo_widget));
+		priv->bonobo_widget = NULL;
+		return FALSE;
+	}
+
+	/* Activate component */
+	bonobo_control_frame_control_activate (
+	       bonobo_widget_get_control_frame (priv->bonobo_widget));
+	gtk_widget_show (GTK_WIDGET (priv->bonobo_widget));
+	
+	return TRUE;
 }
 
 static void
@@ -350,27 +374,75 @@ gw_close_all (void)
 static void
 file_dialog_ok (GtkWidget *widget, gboolean *dialog_result)
 {
-	*dialog_result = TRUE;
+	GtkFileSelection *fsel;
+	const gchar *file_name;
+
+	fsel = GTK_FILE_SELECTION (
+		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
+	file_name = gtk_file_selection_get_filename (fsel);
+
+	if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
+		const gchar *pattern;
+
+		g_message ("!file exists in file_dialog_ok: %s", file_name);
+
+		if (fsel->selection_entry == NULL)
+			return;
+
+		pattern = 
+			gtk_entry_get_text (GTK_ENTRY (fsel->selection_entry));
+		gtk_file_selection_complete (fsel, pattern);
+
+	} else if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
+		/* Change into directory if that's what the user selected */
+		gint name_len;
+		gchar *dir_name;
+
+		name_len = strlen (file_name);
+		if (name_len < 1 || file_name [name_len - 1] != '/') {
+			/* Needed for the file selector */
+			dir_name = g_strconcat (file_name, "/", NULL);
+		} else {
+			dir_name = g_strdup (file_name);
+		}
+
+		gtk_file_selection_set_filename (fsel, dir_name);
+		g_free (dir_name);
+	} else {
+		gtk_widget_hide (GTK_WIDGET (fsel));
+		gtk_main_quit ();
+		*dialog_result = TRUE;
+	}
+}
+
+static void
+file_dialog_cancel (GtkWidget *widget, gpointer user_data)
+{
+	gtk_widget_hide (
+		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
 	gtk_main_quit ();
 }
 
 static guint
 file_dialog_delete_event (GtkWidget *widget, GdkEventAny *event)
 {
+	gtk_widget_hide (widget);
 	gtk_main_quit ();
 	return TRUE;
 }
 
 static gboolean
-gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
+gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 {
 	GtkFileSelection *fsel;
 	gboolean accepted = FALSE;
 
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
+	g_return_val_if_fail (uri, FALSE);
 	g_return_val_if_fail (*uri == NULL, FALSE);
 
 	fsel = GTK_FILE_SELECTION (gtk_file_selection_new (_("Load file")));
+	gtk_file_selection_set_select_multiple (fsel, TRUE); 
 	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW (fsel),
 				      GTK_WINDOW (gpdf_window));
@@ -379,7 +451,7 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
 	g_signal_connect (G_OBJECT (fsel->ok_button), "clicked",
 			  G_CALLBACK (file_dialog_ok), &accepted);
 	g_signal_connect (G_OBJECT (fsel->cancel_button), "clicked",
-			  G_CALLBACK (gtk_main_quit), NULL);
+			  G_CALLBACK (file_dialog_cancel), NULL);
 
 	gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
 
@@ -392,10 +464,26 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
 	gtk_grab_add (GTK_WIDGET (fsel));
 	gtk_main ();
 
-	if (accepted)
-		*uri = gnome_vfs_get_uri_from_local_path (
-			gtk_file_selection_get_filename (fsel));
-	
+	if (accepted) {
+		gchar **files;
+		guint n; 
+
+		files = gtk_file_selection_get_selections (fsel);
+
+		n = 0; 
+		while (files[n]) 
+			n++;
+		*uri = g_new (gchar*, n+1);
+
+		n = 0; 
+		while (files[n]) {
+			(*uri)[n] =
+				gnome_vfs_get_uri_from_local_path (files[n]);
+			n++;
+		}
+		(*uri)[n] = NULL; 
+	}
+
 	gtk_widget_destroy (GTK_WIDGET (fsel));
 
 	return accepted;
@@ -404,16 +492,21 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
 static void
 gw_open_dialog (GPdfWindow *gpdf_window)
 {
-	gchar *uri = NULL;
+	gchar **uri = NULL;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window));
 
 	if (gw_ask_for_uri (gpdf_window, &uri)) {
+		guint n;
+		
 		g_assert (uri != NULL);
 
-		gpdf_uri_input_open_uri (uri_in, uri);
-
-		g_free (uri);
+		n = 0; 
+		while (uri[n]) {
+			gpdf_uri_input_open_uri (uri_in, uri[n]);
+			n++;
+		}
+		g_strfreev (uri); 
 	}
 }
 
@@ -462,22 +555,41 @@ raise_and_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (widget);
 }
 
+static void
+verb_HelpContents_cb (BonoboUIComponent *uic, gpointer user_data,
+		     const char *cname)
+{
+	gnome_help_display ("gpdf.xml",
+			    NULL,
+			    NULL);
+}
+
 static void
 verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 		   const char *cname)
 {
 	static GtkWidget *about;
 
+	gchar *tmp_str;
+
 	static const gchar *authors[] = {
 		N_("Derek B. Noonburg, Xpdf author."),
 		N_("Martin Kretzschmar, GNOME port maintainer."),
 		N_("Michael Meeks, GNOME port creator."),
+		/* please localize as R&eacute;mi (R\303\251mi) */
+		N_("Remi Cohen-Scali."),
 		N_("Miguel de Icaza."),
 		N_("Nat Friedman."),
 		N_("Ravi Pratap."),
 		NULL
 	};
 
+	static const gchar *documenters[] = {
+		N_("Chee Bin HOH <cbhoh@mimos.my>"),
+		N_("Breda McColgan <BredaMcColgan@sun.com>"),
+		NULL
+	};
+
 	static const gchar *translators;
 
 	if (!about) {
@@ -489,14 +601,18 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 #endif
 
 		translators = _("translator_credits-PLEASE_ADD_YOURSELF_HERE");
-		
+
+		tmp_str = g_strconcat ("Copyright \xc2\xa9 1996-2003 ",
+				     _("Glyph & Cog, LLC and authors"),
+				     NULL);
+
 		about = gnome_about_new (
 			_("GNOME PDF Viewer"),
 			VERSION,
-			_("Copyright (C) 1996-2003 Glyph & Cog, LLC and authors"),
+			tmp_str,
 			_("A PDF viewer based on Xpdf"),
 			authors,
-			NULL,
+			documenters,
 			(strcmp (translators,
 				 "translator_credits-PLEASE_ADD_YOURSELF_HERE")
 			 ? translators : NULL),
@@ -505,6 +621,8 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 		g_signal_connect (about, "destroy",
 				  G_CALLBACK (gtk_widget_destroyed),
 				  &about);
+
+		g_free (tmp_str);
 	}
 
 	gtk_widget_show_now (about);
@@ -613,6 +731,7 @@ static BonoboUIVerb gw_verbs [] = {
 	BONOBO_UI_VERB ("FileClose", verb_FileClose_cb),
 	BONOBO_UI_VERB ("FileExit",  verb_FileExit_cb),
 
+	BONOBO_UI_VERB ("HelpContents", verb_HelpContents_cb),
 	BONOBO_UI_VERB ("HelpAbout", verb_HelpAbout_cb),
 
 	BONOBO_UI_VERB_END
